/*
 * Author: Jake G
 * Date: 2025
 * filename: ADC.c
 * description: module_purpose
 */

#ifndef __AVR_ATtiny13A__
#define __AVR_ATtiny13A__
#endif

#include "ADC.h"
#include "RegEdit.h"
#include "avr/io.h"

#define MAX_CHANNEL_NUM 3

static bool IsInvalidChannel(uint8_t adc_chan) {
  if (adc_chan > MAX_CHANNEL_NUM) {
    return true;
  }
  return false;
}

uint8_t ADC_GetChannelPinNum(uint8_t adc_chan) {
  switch (adc_chan) {
  case 0:
    return PB5;
  case 1:
    return PB2;
  case 2:
    return PB4;
  case 3:
    return PB3;
  default:
    return 255; /*return invalid pin num.*/
  }
}

void ADC_Setup(void) {
  // Clear the register, set VCC as ref, ADC0 as channel and
  // leave result right adjusted.
  RegEdit_ClearRegister((void *)&ADMUX);

  // Status & Control register A
  // Turn on the 8Div prescaler
  RegEdit_SetNum((void *)&ADCSRA, (1 << ADPS1) | (1 << ADPS0));

  // Status & Control register B
  // Turn on auto-trigger using time compare/match B.
  RegEdit_AND_Num((void *)&ADCSRB, (1 << ADTS2) | (0 << ADTS1) | (1 << ADTS0));
}

void ADC_Init(uint8_t adc_chan) {

  if (IsInvalidChannel(adc_chan)) {
    return;
  }

  // get the associated pin number.
  uint8_t pin_num = ADC_GetChannelPinNum(adc_chan);

  // set the direction to input
  RegEdit_ClearBit((void *)&DDRB, pin_num);

  // Disable the pull-up resistor
  RegEdit_ClearBit((void *)&PORTB, pin_num);

  // Set the adc mux reg for our channel.

  // Clear the existing mux bits.
  RegEdit_AND_Num((void *)&ADMUX, 0xFC);

  // Set the correct bits
  RegEdit_OR_Num((void *)&ADMUX, adc_chan);
}

void ADC_Enable(void) {
  // Set the enable bit in the CTRLA register
  RegEdit_SetBit((void *)&ADCSRA, ADEN);
}

void ADC_Disable() {
  // Clear the enable ADC flag
  RegEdit_ClearBit((void *)&ADCSRA, ADEN);
}

uint16_t ADC_ReadValue_Impl(void) {

  // start conversion.
  RegEdit_SetBit((void *)&ADCSRA, ADSC);

  /* Wait until ADC conversion done or it times out. */
  uint8_t cycles = 1;
  while (1) {
    if (!RegEdit_IsBitSet((void *)&ADCSRA, ADSC)) {
      break;
    }
    // if It times out return an invalid value for a 12bit ADC.
    else if (cycles >= ADC_WAIT_TIMEOUT) {
      return UINT16_MAX;
    }
    cycles++;
  }

  uint16_t val = ADC;
  return val;
}

// Set the default for the function pointer.
uint16_t (*ADC_ReadValue)(void) = ADC_ReadValue_Impl;
